-------------------------------------------------------------------------------
-- Program Purge                                                             --
-- (C) Copyright 2000 ADALOG                                                 --
-- Author: J-P. Rosen                                                        --
--                                                                           --
-- Rights to use, distribute or modify this program in any way is hereby     --
-- granted, provided this header is kept unchanged in all versions and the   --
-- associated documentation file is distributed unchanged. Additionnal       --
-- headers or documentation may be added.                                    --
-- All modifications must be properly marked as not originating from Adalog. --
-- If you make a valuable addition, please keep us informed by sending a     --
-- message to rosen.adalog@wanadoo.fr                                        --
--                                                                           --
-- ADALOG is providing training, consultancy and expertise in Ada and        --
-- related software engineering techniques. For more info about our services:--
-- ADALOG                   Tel: +33 1 41 24 31 40                           --
-- 19-21 rue du 8 mai 1945  Fax: +33 1 41 24 07 36                           --
-- 94110 ARCUEIL            E-m: rosen.adalog@wanadoo.fr                     --
--                          URL: http://pro.wanadoo.fr/adalog                --
--                                                                           --
-- This package is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY  --
-- or FITNESS FOR A PARTICULAR PURPOSE.                                      --
-------------------------------------------------------------------------------

-- Example program to demonstrate use of OS_Services
-- This program purges backup files
-- Usage:
--   purge [-r] [-i]
-- The program removes any *.bak, *.back, *.backup.*, and *~ files
-- if the option -r is given, purge applies recursively to all subdirectories
-- if the option -i is given, the user is asked for permission before deleting
-- the file.

with OS_Services;      use OS_Services;
with Ada.Text_IO;      use Ada.Text_IO;
with Ada.Command_Line;
procedure Purge is

   -- Command line options:
   Option_I : Boolean := False;
   Option_R : Boolean := False;

   -- Command line parsing:
   Syntax_Error : exception;
   procedure Analyse_Options is
      use Ada.Command_Line;
   begin
      for I in 1..Argument_Count loop
         if Argument (I) = "-i" then
            Option_I := True;
         elsif Argument (I) = "-r" then
            Option_R := True;
         else
            Put_Line ("Unknown option: " & Argument (I));
            raise Syntax_Error;
         end if;
      end loop;
   end Analyse_Options;

   -- A filter to select directories:
   function Directory_Select (List : File_List; Index : Positive) return Boolean is
   begin
      return Property (Is_Directory, List, Index);
   end Directory_Select;
   function Directory_Filter is new Generic_Filter (Directory_Select);

   -- List of files that are considered "backup":
   To_Purge : constant File_List := Build_List ("*.bak *.back *.backup.* *~");

   -- Local variables:
   OK_Delete : Boolean;
   Response  : Character;
begin
   Analyse_Options;

   for I in 1..Length (To_Purge) loop
      if Option_I then
         Put (Property (Expanded, To_Purge, I));
         Put (" ? ");
         Get_Immediate (Response);
         OK_Delete := Response = 'y' or Response = 'Y';
         New_Line;
      else
         OK_Delete := True;
      end if;
      if OK_Delete then
         Delete_File (Property (Expanded, To_Purge, I));
      end if;
   end loop;

   if Option_R then
      -- recurse into subdirectories
      declare
         Dir_List  : constant File_List := Directory_Filter (Build_List ("*"));
      begin
         for I in 1..Length (Dir_List) loop
            -- Since we take the absolute name of the directory, we don't have
            -- to care under which directory the previous incantation left us
            Change_Dir (Property (Expanded, Dir_List, I));

            -- Yes, we can call the main program recursively:
            Purge;
         end loop;
      end;
   end if;
exception
   when Syntax_Error =>
      null;
end Purge;
